@@ -5,7 +5,7 @@ package JSON::Schema::Modern::Document::OpenAPI;
 # ABSTRACT: One OpenAPI v3.1 document
 # KEYWORDS: JSON Schema data validation request response OpenAPI
 
-our $VERSION = '0.044';
+our $VERSION = '0.045';
 
 use 5.020;
 use Moo;
@@ -23,7 +23,7 @@ use List::Util qw(any pairs);
 use Ref::Util 'is_plain_hashref';
 use MooX::HandlesVia;
 use MooX::TypeTiny 0.002002;
-use Types::Standard qw(InstanceOf HashRef Str);
+use Types::Standard qw(InstanceOf HashRef Str Enum);
 use namespace::clean;
 
 extends 'JSON::Schema::Modern::Document';
@@ -57,6 +57,22 @@ has json_schema_dialect => (
   coerce => sub { $_[0]->$_isa('Mojo::URL') ? $_[0] : Mojo::URL->new($_[0]) },
 );
 
+my $reffable_entities = Enum[qw(response parameter example request-body header security-scheme link callbacks path-item)];
+
+# json pointer => entity name
+has entities => (
+  is => 'bare',
+  isa => HashRef[$reffable_entities],
+   handles_via => 'Hash',
+   handles => {
+     _add_entity_location => 'set',
+     get_entity_at_location => 'get',
+  },
+  lazy => 1,
+  default => sub { {} },
+
+);
+
 # operationId => document path
 has operationIds => (
   is => 'bare',
@@ -158,8 +174,13 @@ sub traverse ($self, $evaluator) {
           return 1;
         },
         '$ref' => sub ($data, $schema, $state) {
+          my ($entity) = ($schema->{'$ref'} =~ m{#/\$defs/([^/]+?)(?:-or-reference)?$});
+          $self->_add_entity_location($state->{data_path}, $entity)
+            if $entity and $reffable_entities->check($entity);
+
           push @operation_paths, [ $data->{operationId} => $state->{data_path} ]
             if $schema->{'$ref'} eq '#/$defs/operation' and defined $data->{operationId};
+
           return 1;
         },
       },
@@ -277,7 +298,7 @@ JSON::Schema::Modern::Document::OpenAPI - One OpenAPI v3.1 document
 
 =head1 VERSION
 
-version 0.044
+version 0.045
 
 =head1 SYNOPSIS
 
